@@ -1,12 +1,10 @@
-
 package File::Scan::ClamAV;
 use strict;
 use warnings;
 use File::Find qw(find);
 use IO::Socket;
 
-our $VERSION = "1.92";
-$VERSION = eval $VERSION;
+our $VERSION = '1.95';
 
 =head1 NAME
 
@@ -43,19 +41,23 @@ A port or socket to connect to if you do not wish to use the unix domain socket
 Examples:
 
   my $av = new File::Scan::ClamAV; # Default - uses /tmp/clamd socket
-  
+
   # Use the unix domain socket at /var/sock/clam
   my $av = new File::Scan::ClamAV(port => '/var/sock/clam');
-  
+
   # Use tcp/ip at port 3310
   my $av = new File::Scan::ClamAV(port => 3310);
 
-Note: there is no way to connect to a clamd on another machine. The reason for this is that clamd can only scan local files, so there would not be much point in doing this (unless you had NFS shares). Plus if you are using TCP/IP clamd appears to bind to all adaptors, so it is probably insecure. -ms
+Note: Other than using streamscan, there is no way to connect to a clamd on another machine. The reason for this is that clamd can only scan local files unless using the streamscan method.
 
 =item * find_all
 
 By default the ClamAV clamd service will stop scanning at the first virus it detects. This is useful for performance, but sometimes you want to find all possible viruses in all of the files. To do that, specify a true value for find_all.
 
+=item * host
+
+Set what host to connect to if using TCP. Defaults to localhost. This can be handy to use with the C<streamscan> method to enable sending data to a remote ClamAV daemon.
+
 Examples:
 
   # Stop at first virus
@@ -63,7 +65,7 @@ Examples:
 
   my $av = new File::Scan::ClamAV;
   my ($file, $virus) = $av->scan('/home/bob');
-  
+
 
 
   # Return all viruses
@@ -119,6 +121,7 @@ sub new {
     my (%options) = @_;
     $options{port} ||= '/tmp/clamd';
     $options{find_all} ||= 0;
+    $options{host} ||= 'localhost';
     return bless \%options, $class;
 }
 
@@ -135,7 +138,10 @@ sub ping {
  my $conn = $self->_get_connection || return;
 
  $self->_send($conn, "PING\n");
- chomp(my $response = $conn->getline);
+ my $response = $conn->getline;
+ $response = q{} unless defined $response;
+
+ chomp $response ;
 
  # Run out the buffer?
  1 while (<$conn>);
@@ -185,8 +191,8 @@ This method has been deprecated - use scan() instead
 =cut
 
 sub rawscan {
- warn "The rawscan() method is deprecated - using scan() instead";
- shift->scan(@_);
+ warn 'The rawscan() method is deprecated - using scan() instead';
+ return shift->scan(@_);
 }
 
 =head2 streamscan($data);
@@ -200,32 +206,28 @@ On failure it sets the errstr() error handler.
 =cut
 
 sub streamscan {
- my ($self) = shift;
+ my $self = shift;
+ my $data = shift;
 
- my $data = join '', @_;
+ if(@_){ #don't join unless needed [cpan #78769]
+    $data = join q{},($data,@_);
+ }
 
  $self->_seterrstr;
 
  my $conn = $self->_get_connection || return;
- $self->_send($conn, "STREAM\n");
- chomp(my $response = $conn->getline);
+ $self->_send($conn, "nINSTREAM\n");
+ $self->_send($conn, pack("N", length($data)));
+ $self->_send($conn, $data);
+ $self->_send($conn, pack("N", 0));
+
+ chomp(my $r = $conn->getline);
 
  my @return;
- if($response =~ /^PORT (\d+)/){
-	if((my $c = $self->_get_tcp_connection($1))){
-		$self->_send($c, $data);
-		$c->close;
-
-		chomp(my $r = $conn->getline);
-		if($r =~ /stream: (.+) FOUND/i){
-			@return = ('FOUND', $1);
-		} else {
-			@return = ('OK');
-		}
-	} else {
-		$conn->close;
-		return;
-	}
+ if($r =~ /stream:\ (.+)\ FOUND/ix){
+	@return = ('FOUND', $1);
+ } else {
+	@return = ('OK');
  }
  $conn->close;
  return @return;
@@ -279,39 +281,78 @@ sub errstr {
  return $self->{'.errstr'};
 }
 
+=head2 host()
+
+Return current host used for TCP connections.
+
+If passed an IP or Hostname, will set and return.
+
+=cut
+
+sub host {
+ my ($self, $host) = @_;
+
+ if($host){
+    $self->{host} = $host;
+ }
+
+ return $self->{host};
+}
+
+=head2 port()
+
+Return current port used.
+
+If passed a digit or socket file, will set and return.
+
+Values that contain non-digits will be treated as a local UNIX socket.
+
+=cut
+
+sub port {
+ my ($self, $port) = @_;
+
+ if($port){
+    $self->{port} = $port;
+ }
+
+ return $self->{port};
+}
+
+
 sub _scan {
  my $self = shift;
  my $cmd = shift;
  my $options = {};
 
  if(ref($_[-1]) eq 'HASH') {
-	$options = pop(@_);
+	$options = pop @_;
  }
-    
+
  # Ugh - a bug in clamd makes us do every file
  # on a separate connection! So we will do a File::Find
  # ourselves to get all the files, then do each on
  # a separate connection to the daemon. Hopefully
  # this bug will be fixed and I can remove this horrible
  # hack. -ms
-    
+
  # Files
- my @files = grep { -f $_ } @_;
-    
+ my @files = grep { -f } @_;
+
  # Directories
  for my $dir (@_){
 	next unless -d $dir;
-	find(sub {
+    find({untaint =>1, wanted=>  sub {
 		if(-f $File::Find::name) {
 			push @files, $File::Find::name;
 		}
-	}, $dir);
+	}}, $dir);
  }
 
  if(!@files) {
-	return $self->_seterrstr("scan() requires that you specify a directory or file to scan");
+	return $self->_seterrstr('scan() requires that you specify a directory or file to scan');
  }
-    
+
  my @results;
 
  for(@files){
@@ -328,7 +369,7 @@ sub _scan_shallow {
  my $options = {};
 
  if(ref($_[-1]) eq 'HASH') {
-        $options = pop(@_);
+        $options = pop @_;
  }
 
  my @dirs = @_;
@@ -339,20 +380,20 @@ sub _scan_shallow {
 	$self->_send($conn, "$cmd $file\n");
 
 	for my $result ($conn->getline){
-		chomp($result);
+		chomp $result;
 
-		my @result = split(/\s/, $result);
+		my @result = split /\s/x, $result;
 
 		chomp(my $code = pop @result);
-		if($code !~ /^(?:ERROR|FOUND|OK)$/){
+		if($code !~ /^(?:ERROR|FOUND|OK)$/x){
 			$conn->close;
 
-			return $self->_seterrstr("Unknown response code from ClamAV service: $code - " . join(" ", @result));
+			return $self->_seterrstr("Unknown response code from ClamAV service: $code - " . join q{ }, @result);
 		}
 
 		my $virus = pop @result;
-		my $file = join(" ", @result);
-		$file =~ s/:$//g;
+		my $file = join q{ }, @result;
+		$file =~ s/:$//gx;
 
 		if($code eq 'ERROR'){
 			$conn->close;
@@ -375,14 +416,19 @@ sub _seterrstr {
  return;
 }
 
+#TODO: set a timeout and fork so we don't
+#get stuck waiting too long on clamd?
 sub _send {
- my ($self, $fh, $data) = @_;
- return syswrite $fh, $data, length($data);
+ my $self = shift;
+ my $fh = shift;
+
+ #use alias to save mem [cpan #78769]
+ return syswrite $fh, $_[0], length $_[0];
 }
 
 sub _get_connection {
  my ($self) = @_;
- if($self->{port} =~ /\D/){
+ if($self->{port} =~ /\D/x){
 	return $self->_get_unix_connection;
  } else {
 	return $self->_get_tcp_connection;
@@ -391,15 +437,16 @@ sub _get_connection {
 
 sub _get_tcp_connection {
  my ($self, $port) = @_;
+ my $host = $self->host;
  $port ||= $self->{port};
 
  return IO::Socket::INET->new(
-	PeerAddr	=> 'localhost',
-	PeerPort	=> $port,
-	Proto		=> 'tcp',
-	Type		=> SOCK_STREAM,
-	Timeout		=> 10
- ) || $self->_seterrstr("Cannot connect to 'localhost:$port': $@");
+        PeerAddr => $host,
+        PeerPort => $port,
+        Proto    => 'tcp',
+        Type     => SOCK_STREAM,
+        Timeout  => 10,
+ ) || $self->_seterrstr("Cannot connect to '$host:$port': $@");
 }
 
 sub _get_unix_connection {
@@ -413,6 +460,12 @@ sub _get_unix_connection {
 1;
 __END__
 
+=head1 CAVEATS
+
+=head2 Supported Operating Systems
+
+Currenly only Linux-like systems are supported. Patches are welcome.
+
 =head1 AUTHOR
 
 Colin Faber <cfaber@fpsn.net> All Rights Reserved.
